@@ -2,307 +2,325 @@ package File::HomeDir;
 
 # See POD at end for documentation
 
-use 5.008003;
+use 5.00503;
 use strict;
-use warnings;
 use Carp        ();
 use Config      ();
 use File::Spec  ();
 use File::Which ();
 
 # Globals
-use vars qw{$VERSION @EXPORT @EXPORT_OK $IMPLEMENTED_BY};    ## no critic qw(AutomaticExportation)
-use base qw(Exporter);
+use vars qw{$VERSION @ISA @EXPORT @EXPORT_OK $IMPLEMENTED_BY};
+BEGIN {
+	$VERSION = '1.002';
 
-BEGIN
-{
-    $VERSION = '1.003_002';
+	# Inherit manually
+	require Exporter;
+	@ISA       = qw{ Exporter };
+	@EXPORT    = qw{ home     };
+	@EXPORT_OK = qw{
+		home
+		my_home
+		my_desktop
+		my_documents
+		my_music
+		my_pictures
+		my_videos
+		my_data
+		my_dist_config
+		my_dist_data
+		users_home
+		users_desktop
+		users_documents
+		users_music
+		users_pictures
+		users_videos
+		users_data
+	};
 
-    # Inherit manually
-    require Exporter;
-    @EXPORT    = qw{home};
-    @EXPORT_OK = qw{
-      home
-      my_home
-      my_desktop
-      my_documents
-      my_music
-      my_pictures
-      my_videos
-      my_data
-      my_dist_config
-      my_dist_data
-      users_home
-      users_desktop
-      users_documents
-      users_music
-      users_pictures
-      users_videos
-      users_data
-    };
+	# %~ doesn't need (and won't take) exporting, as it's a magic
+	# symbol name that's always looked for in package 'main'.
 }
 
 # Inlined Params::Util functions
-sub _CLASS ($)    ## no critic qw(SubroutinePrototypes)
-{
-    (defined $_[0] and not ref $_[0] and $_[0] =~ m/^[^\W\d]\w*(?:::\w+)*\z/s) ? $_[0] : undef;
+sub _CLASS ($) {
+	(defined $_[0] and ! ref $_[0] and $_[0] =~ m/^[^\W\d]\w*(?:::\w+)*\z/s) ? $_[0] : undef;
 }
-
-sub _DRIVER ($$)    ## no critic qw(SubroutinePrototypes)
-{
-    (defined _CLASS($_[0]) and eval "require $_[0]; 1" and $_[0]->isa($_[1]) and $_[0] ne $_[1]) ? $_[0] : undef;
+sub _DRIVER ($$) {
+	(defined _CLASS($_[0]) and eval "require $_[0];" and ! $@ and $_[0]->isa($_[1]) and $_[0] ne $_[1]) ? $_[0] : undef;
 }
 
 # Platform detection
-if ($IMPLEMENTED_BY)
-{
-    # Allow for custom HomeDir classes
-    # Leave it as the existing value
+if ( $IMPLEMENTED_BY ) {
+	# Allow for custom HomeDir classes
+	# Leave it as the existing value
+} elsif ( $^O eq 'MSWin32' ) {
+	# All versions of Windows
+	$IMPLEMENTED_BY = 'File::HomeDir::Windows';
+} elsif ( $^O eq 'darwin') {
+	# 1st: try Mac::SystemDirectory by chansen
+	if ( eval { require Mac::SystemDirectory; 1 } ) {
+		$IMPLEMENTED_BY = 'File::HomeDir::Darwin::Cocoa';
+	} elsif ( eval { require Mac::Files; 1 } ) {
+		# 2nd try Mac::Files: Carbon - unmaintained since 2006 except some 64bit fixes
+		$IMPLEMENTED_BY = 'File::HomeDir::Darwin::Carbon';
+	} else {
+		# 3rd: fallback: pure perl
+		$IMPLEMENTED_BY = 'File::HomeDir::Darwin';
+	}
+} elsif ( $^O eq 'MacOS' ) {
+	# Legacy Mac OS
+	$IMPLEMENTED_BY = 'File::HomeDir::MacOS9';
+} elsif ( File::Which::which('xdg-user-dir') ) {
+	# freedesktop unixes
+	$IMPLEMENTED_BY = 'File::HomeDir::FreeDesktop';
+} else {
+	# Default to Unix semantics
+	$IMPLEMENTED_BY = 'File::HomeDir::Unix';
 }
-elsif ($^O eq 'MSWin32')
-{
-    # All versions of Windows
-    $IMPLEMENTED_BY = 'File::HomeDir::Windows';
-}
-elsif ($^O eq 'darwin')
-{
-    # 1st: try Mac::SystemDirectory by chansen
-    if (eval "require Mac::SystemDirectory; 1")
-    {
-        $IMPLEMENTED_BY = 'File::HomeDir::Darwin::Cocoa';
-    }
-    elsif (eval "require Mac::Files; 1")
-    {
-        # 2nd try Mac::Files: Carbon - unmaintained since 2006 except some 64bit fixes
-        $IMPLEMENTED_BY = 'File::HomeDir::Darwin::Carbon';
-    }
-    else
-    {
-        # 3rd: fallback: pure perl
-        $IMPLEMENTED_BY = 'File::HomeDir::Darwin';
-    }
-}
-elsif ($^O eq 'MacOS')
-{
-    # Legacy Mac OS
-    $IMPLEMENTED_BY = 'File::HomeDir::MacOS9';
-}
-elsif (File::Which::which('xdg-user-dir'))
-{
-    # freedesktop unixes
-    $IMPLEMENTED_BY = 'File::HomeDir::FreeDesktop';
-}
-else
-{
-    # Default to Unix semantics
-    $IMPLEMENTED_BY = 'File::HomeDir::Unix';
+unless ( _DRIVER($IMPLEMENTED_BY, 'File::HomeDir::Driver') ) {
+	Carp::croak("Missing or invalid File::HomeDir driver $IMPLEMENTED_BY");
 }
 
-unless (_DRIVER($IMPLEMENTED_BY, 'File::HomeDir::Driver'))
-{
-    Carp::croak("Missing or invalid File::HomeDir driver $IMPLEMENTED_BY");
-}
+
+
+
 
 #####################################################################
 # Current User Methods
 
-sub my_home
-{
-    $IMPLEMENTED_BY->my_home;
+sub my_home {
+	$IMPLEMENTED_BY->my_home;
 }
 
-sub my_desktop
-{
-    $IMPLEMENTED_BY->can('my_desktop')
-      ? $IMPLEMENTED_BY->my_desktop
-      : Carp::croak("The my_desktop method is not implemented on this platform");
+sub my_desktop {
+	$IMPLEMENTED_BY->can('my_desktop')
+		? $IMPLEMENTED_BY->my_desktop
+		: Carp::croak("The my_desktop method is not implemented on this platform");
 }
 
-sub my_documents
-{
-    $IMPLEMENTED_BY->can('my_documents')
-      ? $IMPLEMENTED_BY->my_documents
-      : Carp::croak("The my_documents method is not implemented on this platform");
+sub my_documents {
+	$IMPLEMENTED_BY->can('my_documents')
+		? $IMPLEMENTED_BY->my_documents
+		: Carp::croak("The my_documents method is not implemented on this platform");
 }
 
-sub my_music
-{
-    $IMPLEMENTED_BY->can('my_music')
-      ? $IMPLEMENTED_BY->my_music
-      : Carp::croak("The my_music method is not implemented on this platform");
+sub my_music {
+	$IMPLEMENTED_BY->can('my_music')
+		? $IMPLEMENTED_BY->my_music
+		: Carp::croak("The my_music method is not implemented on this platform");
 }
 
-sub my_pictures
-{
-    $IMPLEMENTED_BY->can('my_pictures')
-      ? $IMPLEMENTED_BY->my_pictures
-      : Carp::croak("The my_pictures method is not implemented on this platform");
+sub my_pictures {
+	$IMPLEMENTED_BY->can('my_pictures')
+		? $IMPLEMENTED_BY->my_pictures
+		: Carp::croak("The my_pictures method is not implemented on this platform");
 }
 
-sub my_videos
-{
-    $IMPLEMENTED_BY->can('my_videos')
-      ? $IMPLEMENTED_BY->my_videos
-      : Carp::croak("The my_videos method is not implemented on this platform");
+sub my_videos {
+	$IMPLEMENTED_BY->can('my_videos')
+		? $IMPLEMENTED_BY->my_videos
+		: Carp::croak("The my_videos method is not implemented on this platform");
 }
 
-sub my_data
-{
-    $IMPLEMENTED_BY->can('my_data')
-      ? $IMPLEMENTED_BY->my_data
-      : Carp::croak("The my_data method is not implemented on this platform");
+sub my_data {
+	$IMPLEMENTED_BY->can('my_data')
+		? $IMPLEMENTED_BY->my_data
+		: Carp::croak("The my_data method is not implemented on this platform");
 }
 
-sub my_dist_data
-{
-    my $params = ref $_[-1] eq 'HASH' ? pop : {};
-    my $dist = pop or Carp::croak("The my_dist_data method requires an argument");
-    my $data = my_data();
 
-    # If datadir is not defined, there's nothing we can do: bail out
-    # and return nothing...
-    return undef unless defined $data;
+sub my_dist_data {
+	my $params = ref $_[-1] eq 'HASH' ? pop : {};
+	my $dist   = pop or Carp::croak("The my_dist_data method requires an argument");
+	my $data   = my_data();
 
-    # On traditional unixes, hide the top-level directory
-    my $var =
-      $data eq home()
-      ? File::Spec->catdir($data, '.perl', 'dist', $dist)
-      : File::Spec->catdir($data, 'Perl',  'dist', $dist);
+        # If datadir is not defined, there's nothing we can do: bail out
+        # and return nothing...	
+	return undef unless defined $data;
 
-    # directory exists: return it
-    return $var if -d $var;
+        # On traditional unixes, hide the top-level directory
+	my $var = $data eq home()
+		? File::Spec->catdir( $data, '.perl', 'dist', $dist )
+		: File::Spec->catdir( $data, 'Perl',  'dist', $dist );
 
-    # directory doesn't exist: check if we need to create it...
-    return undef unless $params->{create};
+	# directory exists: return it
+	return $var if -d $var;
 
-    # user requested directory creation
-    require File::Path;
-    File::Path::mkpath($var);
-    return $var;
+	# directory doesn't exist: check if we need to create it...
+	return undef unless $params->{create};
+
+	# user requested directory creation
+	require File::Path;
+	File::Path::mkpath( $var );
+	return $var;
 }
 
-sub my_dist_config
-{
-    my $params = ref $_[-1] eq 'HASH' ? pop : {};
-    my $dist = pop or Carp::croak("The my_dist_config method requires an argument");
+sub my_dist_config {
+	my $params = ref $_[-1] eq 'HASH' ? pop : {};
+	my $dist   = pop or Carp::croak("The my_dist_config method requires an argument");
 
-    # not all platforms support a specific my_config() method
-    my $config =
-        $IMPLEMENTED_BY->can('my_config')
-      ? $IMPLEMENTED_BY->my_config
-      : $IMPLEMENTED_BY->my_documents;
+	# not all platforms support a specific my_config() method
+	my $config = $IMPLEMENTED_BY->can('my_config')
+		? $IMPLEMENTED_BY->my_config
+		: $IMPLEMENTED_BY->my_documents;
 
-    # If neither configdir nor my_documents is defined, there's
-    # nothing we can do: bail out and return nothing...
-    return undef unless defined $config;
+	# If neither configdir nor my_documents is defined, there's
+	# nothing we can do: bail out and return nothing...	
+	return undef unless defined $config;
 
-    # On traditional unixes, hide the top-level dir
-    my $etc =
-      $config eq home()
-      ? File::Spec->catdir($config, '.perl', $dist)
-      : File::Spec->catdir($config, 'Perl',  $dist);
+	# On traditional unixes, hide the top-level dir
+	my $etc = $config eq home()
+		? File::Spec->catdir( $config, '.perl', $dist )
+		: File::Spec->catdir( $config, 'Perl',  $dist );
 
-    # directory exists: return it
-    return $etc if -d $etc;
+	# directory exists: return it
+	return $etc if -d $etc;
 
-    # directory doesn't exist: check if we need to create it...
-    return undef unless $params->{create};
+	# directory doesn't exist: check if we need to create it...
+	return undef unless $params->{create};
 
-    # user requested directory creation
-    require File::Path;
-    File::Path::mkpath($etc);
-    return $etc;
+	# user requested directory creation
+	require File::Path;
+	File::Path::mkpath( $etc );
+	return $etc;
 }
 
+
+
+
 #####################################################################
 # General User Methods
 
-sub users_home
-{
-    $IMPLEMENTED_BY->can('users_home')
-      ? $IMPLEMENTED_BY->users_home($_[-1])
-      : Carp::croak("The users_home method is not implemented on this platform");
+sub users_home {
+	$IMPLEMENTED_BY->can('users_home')
+		? $IMPLEMENTED_BY->users_home( $_[-1] )
+		: Carp::croak("The users_home method is not implemented on this platform");
 }
 
-sub users_desktop
-{
-    $IMPLEMENTED_BY->can('users_desktop')
-      ? $IMPLEMENTED_BY->users_desktop($_[-1])
-      : Carp::croak("The users_desktop method is not implemented on this platform");
+sub users_desktop {
+	$IMPLEMENTED_BY->can('users_desktop')
+		? $IMPLEMENTED_BY->users_desktop( $_[-1] )
+		: Carp::croak("The users_desktop method is not implemented on this platform");
 }
 
-sub users_documents
-{
-    $IMPLEMENTED_BY->can('users_documents')
-      ? $IMPLEMENTED_BY->users_documents($_[-1])
-      : Carp::croak("The users_documents method is not implemented on this platform");
+sub users_documents {
+	$IMPLEMENTED_BY->can('users_documents')
+		? $IMPLEMENTED_BY->users_documents( $_[-1] )
+		: Carp::croak("The users_documents method is not implemented on this platform");
 }
 
-sub users_music
-{
-    $IMPLEMENTED_BY->can('users_music')
-      ? $IMPLEMENTED_BY->users_music($_[-1])
-      : Carp::croak("The users_music method is not implemented on this platform");
+sub users_music {
+	$IMPLEMENTED_BY->can('users_music')
+		? $IMPLEMENTED_BY->users_music( $_[-1] )
+		: Carp::croak("The users_music method is not implemented on this platform");
 }
 
-sub users_pictures
-{
-    $IMPLEMENTED_BY->can('users_pictures')
-      ? $IMPLEMENTED_BY->users_pictures($_[-1])
-      : Carp::croak("The users_pictures method is not implemented on this platform");
+sub users_pictures {
+	$IMPLEMENTED_BY->can('users_pictures')
+		? $IMPLEMENTED_BY->users_pictures( $_[-1] )
+		: Carp::croak("The users_pictures method is not implemented on this platform");
 }
 
-sub users_videos
-{
-    $IMPLEMENTED_BY->can('users_videos')
-      ? $IMPLEMENTED_BY->users_videos($_[-1])
-      : Carp::croak("The users_videos method is not implemented on this platform");
+sub users_videos {
+	$IMPLEMENTED_BY->can('users_videos')
+		? $IMPLEMENTED_BY->users_videos( $_[-1] )
+		: Carp::croak("The users_videos method is not implemented on this platform");
 }
 
-sub users_data
-{
-    $IMPLEMENTED_BY->can('users_data')
-      ? $IMPLEMENTED_BY->users_data($_[-1])
-      : Carp::croak("The users_data method is not implemented on this platform");
+sub users_data {
+	$IMPLEMENTED_BY->can('users_data')
+		? $IMPLEMENTED_BY->users_data( $_[-1] )
+		: Carp::croak("The users_data method is not implemented on this platform");
 }
 
+
+
+
+
 #####################################################################
 # Legacy Methods
 
 # Find the home directory of an arbitrary user
-sub home (;$)    ## no critic qw(SubroutinePrototypes)
-{
-    # Allow to be called as a method
-    if ($_[0] and $_[0] eq 'File::HomeDir')
-    {
-        shift();
-    }
+sub home (;$) {
+	# Allow to be called as a method
+	if ( $_[0] and $_[0] eq 'File::HomeDir' ) {
+		shift();
+	}
 
-    # No params means my home
-    return my_home() unless @_;
+	# No params means my home
+	return my_home() unless @_;
 
-    # Check the param
-    my $name = shift;
-    if (!defined $name)
-    {
-        Carp::croak("Can't use undef as a username");
-    }
-    if (!length $name)
-    {
-        Carp::croak("Can't use empty-string (\"\") as a username");
-    }
+	# Check the param
+	my $name = shift;
+	if ( ! defined $name ) {
+		Carp::croak("Can't use undef as a username");
+	}
+	if ( ! length $name ) {
+		Carp::croak("Can't use empty-string (\"\") as a username");
+	}
 
-    # A dot also means my home
-    ### Is this meant to mean File::Spec->curdir?
-    if ($name eq '.')
-    {
-        return my_home();
-    }
+	# A dot also means my home
+	### Is this meant to mean File::Spec->curdir?
+	if ( $name eq '.' ) {
+		return my_home();
+	}
 
-    # Now hand off to the implementor
-    $IMPLEMENTED_BY->users_home($name);
+	# Now hand off to the implementor
+	$IMPLEMENTED_BY->users_home($name);
 }
 
+
+
+
+
+#####################################################################
+# Tie-Based Interface
+
+# Okay, things below this point get scary
+
+CLASS: {
+	# Make the class for the %~ tied hash:
+	package File::HomeDir::TIE;
+
+	# Make the singleton object.
+	# (We don't use the hash for anything, though)
+	### THEN WHY MAKE IT???
+	my $SINGLETON = bless {};
+
+	sub TIEHASH { $SINGLETON }
+
+	sub FETCH {
+		# Catch a bad username
+		unless ( defined $_[1] ) {
+			Carp::croak("Can't use undef as a username");
+		}
+
+		# Get our homedir
+		unless ( length $_[1] ) {
+			return File::HomeDir::my_home();
+		}
+
+		# Get a named user's homedir
+		Carp::carp("The tied %~ hash has been deprecated");
+		return File::HomeDir::home($_[1]);
+	}
+
+	sub STORE    { _bad('STORE')    }
+	sub EXISTS   { _bad('EXISTS')   }
+	sub DELETE   { _bad('DELETE')   }
+	sub CLEAR    { _bad('CLEAR')    }
+	sub FIRSTKEY { _bad('FIRSTKEY') }
+	sub NEXTKEY  { _bad('NEXTKEY')  }
+
+	sub _bad ($) {
+		Carp::croak("You can't $_[0] with the %~ hash")
+	}
+}
+
+# Do the actual tie of the global %~ variable
+tie %~, 'File::HomeDir::TIE';
+
 1;
 
 __END__
@@ -315,13 +333,6 @@ __END__
 
 File::HomeDir - Find your home and other directories on any platform
 
-=begin html
-
-<a href="https://travis-ci.org/perl5-utils/File-HomeDir"><img src="https://travis-ci.org/perl5-utils/File-HomeDir.svg?branch=master" alt="Travis CI"/></a>
-<a href='https://coveralls.io/github/perl5-utils/File-HomeDir?branch=master'><img src='https://coveralls.io/repos/github/perl5-utils/File-HomeDir/badge.svg?branch=master' alt='Coverage Status'/></a>
-
-=end html
-
 =head1 SYNOPSIS
 
   use File::HomeDir;
@@ -349,13 +360,31 @@ File::HomeDir - Find your home and other directories on any platform
 =head1 DESCRIPTION
 
 B<File::HomeDir> is a module for locating the directories that are "owned"
-by a user (typically your user) and to solve the various issues that arise
+by a user (typicaly your user) and to solve the various issues that arise
 trying to find them consistently across a wide variety of platforms.
 
 The end result is a single API that can find your resources on any platform,
 making it relatively trivial to create Perl software that works elegantly
 and correctly no matter where you run it.
 
+This module provides two main interfaces.
+
+The first is a modern L<File::Spec>-style interface with a consistent
+OO API and different implementation modules to support various
+platforms. You are B<strongly> recommended to use this interface.
+
+The second interface is for legacy support of the original 0.07 interface
+that exported a C<home()> function by default and tied the C<%~> variable.
+
+It is generally not recommended that you use this interface, but due to
+back-compatibility reasons they will remain supported until at least 2010.
+
+The C<%~> interface has been deprecated. Documentation was removed in 2009,
+Unit test were removed in 2011, usage will issue warnings from 2012, and the
+interface will be removed entirely in 2015  (in line with the general Perl
+toolchain convention of a 10 year support period for legacy APIs that
+are potentially or actually in common use).
+
 =head2 Platform Neutrality
 
 In the Unix world, many different types of data can be mixed together
@@ -419,13 +448,13 @@ does not have a home directory, or dies on error.
 The C<my_desktop> method takes no arguments and returns the "desktop"
 directory for the current user.
 
-Due to the diversity and complexity of implementations required to deal with
+Due to the diversity and complexity of implementions required to deal with
 implementing the required functionality fully and completely, the
 C<my_desktop> method may or may not be implemented on each platform.
 
 That said, I am extremely interested in code to implement C<my_desktop> on
 Unix, as long as it is capable of dealing (as the Windows implementation
-does) with internationalization. It should also avoid false positive
+does) with internationalisation. It should also avoid false positive
 results by making sure it only returns the appropriate directories for the
 appropriate platforms.
 
@@ -479,7 +508,7 @@ does not have a suitable directory, or dies on error.
 =head2 my_data
 
 The C<my_data> method takes no arguments and returns the directory where
-local applications should store their internal data for the current
+local applications should stored their internal data for the current
 user.
 
 Generally an application would create a subdirectory such as C<.foo>,
@@ -510,7 +539,7 @@ internal configuration.
 
 The base directory will be either C<my_config> if the platform supports
 it, or C<my_documents> otherwise. The subdirectory itself will be 
-C<BASE/Perl/Dist-Name>. If the base directory is the user's home directory,
+C<BASE/Perl/Dist-Name>. If the base directory is the user's homedir,
 C<my_dist_config> will be in C<~/.perl/Dist-Name> (and thus be hidden on
 all Unixes).
 
@@ -547,7 +576,7 @@ This directory will be of course a subdirectory of C<my_data>. Platforms
 supporting data-specific directories will use
 C<DATA_DIR/perl/dist/Dist-Name> following the common
 "DATA/vendor/application" pattern. If the C<my_data> directory is the
-user's home directory, C<my_dist_data> will be in C<~/.perl/dist/Dist-Name>
+user's homedir, C<my_dist_data> will be in C<~/.perl/dist/Dist-Name>
 (and thus be hidden on all Unixes).
 
 The optional last argument is a hash reference to tweak the method
@@ -569,7 +598,7 @@ Defaults to false, meaning no automatic creation of directory.
 
   $home = File::HomeDir->users_home('foo');
 
-The C<users_home> method takes a single parameter and is used to locate the
+The C<users_home> method takes a single param and is used to locate the
 parent home/profile directory for an identified user on the system.
 
 While most of the time this identifier would be some form of user name,
@@ -593,34 +622,6 @@ does not have a documents directory, or dies on error.
 Returns the directory path as a string, C<undef> if that user
 does not have a data directory, or dies on error.
 
-=head2 users_desktop
-
-  $docs = File::HomeDir->users_desktop('foo');
-
-Returns the directory path as a string, C<undef> if that user
-does not have a desktop directory, or dies on error.
-
-=head2 users_music
-
-  $docs = File::HomeDir->users_music('foo');
-
-Returns the directory path as a string, C<undef> if that user
-does not have a music directory, or dies on error.
-
-=head2 users_pictures
-
-  $docs = File::HomeDir->users_pictures('foo');
-
-Returns the directory path as a string, C<undef> if that user
-does not have a pictures directory, or dies on error.
-
-=head2 users_videos
-
-  $docs = File::HomeDir->users_videos('foo');
-
-Returns the directory path as a string, C<undef> if that user
-does not have a videos directory, or dies on error.
-
 =head1 FUNCTIONS
 
 =head2 home
@@ -637,7 +638,7 @@ use the newer method-based interface above.
 
 Returns the directory path to a named user's home/profile directory.
 
-If provided no parameter, returns the directory path to the current user's
+If provided no param, returns the directory path to the current user's
 home/profile directory.
 
 =head1 TO DO
